@@ -1,4 +1,5 @@
 # Copyright (C) 2008 Red Hat, Inc.
+# Copyright (C) 2016 Sam Parkinson <sam@sam.today>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -15,9 +16,15 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
-"""
-STABLE.
-"""
+'''
+The bundle builder is a build system for Sugar activities.  Usually, it
+is setup by creating a `setup.py` file in the project with the following::
+
+    #!/usr/bin/env python
+
+    from sugar3.activity import bundlebuilder
+    bundlebuilder.start()
+'''
 
 import argparse
 import operator
@@ -31,7 +38,11 @@ import subprocess
 import re
 import gettext
 import logging
+from glob import glob
 from fnmatch import fnmatch
+from ConfigParser import ConfigParser
+import xml.etree.cElementTree as ET
+from HTMLParser import HTMLParser
 
 from sugar3 import env
 from sugar3.bundle.activitybundle import ActivityBundle
@@ -80,6 +91,7 @@ class Config(object):
         self.xo_name = None
         self.tar_name = None
         self.summary = None
+        self.description = None
 
         self.update()
 
@@ -90,6 +102,7 @@ class Config(object):
         self.activity_name = bundle.get_name()
         self.bundle_id = bundle.get_bundle_id()
         self.summary = bundle.get_summary()
+        self.description = bundle.get_description()
         self.bundle_name = reduce(operator.add, self.activity_name.split())
         self.bundle_root_dir = self.bundle_name + '.activity'
         self.tar_root_dir = '%s-%s' % (self.bundle_name, self.version)
@@ -121,7 +134,7 @@ class Builder(object):
         if os.path.exists(self.locale_dir):
             shutil.rmtree(self.locale_dir)
 
-        for f in os.listdir(po_dir):
+        for f in sorted(os.listdir(po_dir)):
             if not f.endswith('.po') or f == 'pseudo.po':
                 continue
 
@@ -144,6 +157,8 @@ class Builder(object):
             cat = gettext.GNUTranslations(open(mo_file, 'r'))
             translated_name = cat.gettext(self.config.activity_name)
             translated_summary = cat.gettext(self.config.summary)
+            if translated_summary is None:
+                translated_summary = ''
             if translated_summary.find('\n') > -1:
                 translated_summary = translated_summary.replace('\n', '')
                 logging.warn(
@@ -168,12 +183,15 @@ class Packager(object):
         if not os.path.exists(self.config.dist_dir):
             os.mkdir(self.config.dist_dir)
 
-    def get_files_in_git(self):
+    def get_files_in_git(self, root=None):
+        if root is None:
+            root = self.config.source_dir
+
         git_ls = None
         try:
             git_ls = subprocess.Popen(['git', 'ls-files'],
                                       stdout=subprocess.PIPE,
-                                      cwd=self.config.source_dir)
+                                      cwd=root)
         except OSError:
             logging.warn('Packager: git is not installed, '
                          'fall back to filtered list')
@@ -196,7 +214,14 @@ class Packager(object):
                             ignore = True
                             break
                     if not ignore:
-                        files.append(line)
+                        sub_path = os.path.join(root, line)
+                        if os.path.isdir(sub_path) \
+                           and os.path.exists(os.path.join(sub_path, '.git')):
+                            sub_list = self.get_files_in_git(sub_path)
+                            for f in sub_list:
+                                files.append(os.path.join(line, f))
+                        else:
+                            files.append(line)
 
                 for pattern in IGNORE_FILES:
                     files = [f for f in files if not fnmatch(f, pattern)]
@@ -253,7 +278,7 @@ class Installer(Packager):
         Packager.__init__(self, builder.config)
         self.builder = builder
 
-    def install(self, prefix, install_mime=True):
+    def install(self, prefix, install_mime=True, install_desktop_file=True):
         self.builder.build()
 
         activity_path = os.path.join(prefix, 'share', 'sugar', 'activities',
@@ -288,6 +313,110 @@ class Installer(Packager):
         if install_mime:
             self.config.bundle.install_mime_type(self.config.source_dir)
 
+        if install_desktop_file:
+            self._install_desktop_file(prefix, activity_path)
+            self._generate_appdata(prefix, activity_path)
+
+    def _install_desktop_file(self, prefix, activity_path):
+        cp = ConfigParser()
+        section = 'Desktop Entry'
+        cp.add_section(section)
+        cp.optionxform = str  # Allow CamelCase entries
+
+        # Get it from the activity.info for the non-translated version
+        info = ConfigParser()
+        info.read(os.path.join(activity_path, 'activity', 'activity.info'))
+        cp.set(section, 'Name', info.get('Activity', 'name'))
+        if info.has_option('Activity', 'summary'):
+            cp.set(section, 'Comment', info.get('Activity', 'summary'))
+
+        for path in sorted(glob(os.path.join(activity_path, 'locale',
+                                             '*', 'activity.linfo'))):
+            locale = path.split(os.path.sep)[-2]
+            info = ConfigParser()
+            info.read(path)
+            if info.has_option('Activity', 'name'):
+                cp.set(section, 'Name[{}]'.format(locale),
+                       info.get('Activity', 'name'))
+            if info.has_option('Activity', 'summary'):
+                cp.set(section, 'Comment[{}]'.format(locale),
+                       info.get('Activity', 'summary'))
+
+        cp.set(section, 'Terminal', 'false')
+        cp.set(section, 'Type', 'Application')
+        cp.set(section, 'Categories', 'Education;')
+        cp.set(section, 'Icon', os.path.join(
+            activity_path, 'activity', self.config.bundle.get_icon_filename()))
+        cp.set(section, 'Exec', self.config.bundle.get_command())
+        cp.set(section, 'Path', activity_path)  # Path == CWD for running
+
+        name = '{}.activity.desktop'.format(self.config.bundle_id)
+        path = os.path.join(prefix, 'share', 'applications', name)
+        if not os.path.isdir(os.path.dirname(path)):
+            os.makedirs(os.path.dirname(path))
+        with open(path, 'w') as f:
+            cp.write(f)
+
+    def _generate_appdata(self, prefix, activity_path):
+        info = ConfigParser()
+        info.read(os.path.join(activity_path, 'activity', 'activity.info'))
+
+        required_fields = ['metadata_license', 'license', 'name', 'icon',
+                           'description']
+        for name in required_fields:
+            if not info.has_option('Activity', name):
+                print('[WARNING] Activity needs more metadata for AppStream '
+                      'file')
+                print('  Without an AppStream file, the activity will NOT '
+                      'show in software stores!')
+                print('  Please `pydoc sugar3.activity.bundlebuilder` for'
+                      'more info')
+                return
+
+        # See https://www.freedesktop.org/software/appstream/docs/
+        root = ET.Element('component', type='desktop')
+        ET.SubElement(root, 'project_group').text = 'Sugar'
+        ET.SubElement(root, 'translation', type='gettext').text = \
+            self.config.bundle_id
+        ET.SubElement(root, 'id').text = \
+            self.config.bundle_id + '.activity.desktop'
+        desc = ET.fromstring('<description>{}</description>'.format(
+            info.get('Activity', 'description')))
+        root.append(desc)
+
+        copy_pairs = [('metadata_license', 'metadata_license'),
+                      ('license', 'project_license'),
+                      ('summary', 'summary'),
+                      ('name', 'name')]
+        for key, ename in copy_pairs:
+            ET.SubElement(root, ename).text = info.get('Activity', key)
+
+        if info.has_option('Activity', 'screenshots'):
+            screenshots = info.get('Activity', 'screenshots').split()
+            ss_root = ET.SubElement(root, 'screenshots')
+            for i, screenshot in enumerate(screenshots):
+                e = ET.SubElement(ss_root, 'screenshot')
+                if i == 0:
+                    e.set('type', 'default')
+                ET.SubElement(e, 'image').text = screenshot
+
+        if info.has_option('Activity', 'url'):
+            ET.SubElement(root, 'url', type='homepage').text = \
+                info.get('Activity', 'url')
+        if info.has_option('Activity', 'repository_url'):
+            ET.SubElement(root, 'url', type='bugtracker').text = \
+                info.get('Activity', 'repository_url')
+        elif info.has_option('Activity', 'repository'):
+            ET.SubElement(root, 'url', type='bugtracker').text = \
+                info.get('Activity', 'repository')
+
+        path = os.path.join(prefix, 'share', 'metainfo',
+                            self.config.bundle_id + '.appdata.xml')
+        if not os.path.isdir(os.path.dirname(path)):
+            os.makedirs(os.path.dirname(path))
+        tree = ET.ElementTree(root)
+        tree.write(path, encoding='UTF-8')
+
 
 def cmd_check(config, options):
     """Run tests for the activity"""
@@ -350,8 +479,11 @@ def cmd_dev(config, options):
 
 def cmd_dist_xo(config, options):
     """Create a xo bundle package"""
+    no_fail = False
+    if options is not None:
+        no_fail = options.no_fail
 
-    packager = XOPackager(Builder(config, options.no_fail))
+    packager = XOPackager(Builder(config, no_fail))
     packager.package()
 
 
@@ -374,7 +506,11 @@ def cmd_install(config, options):
     """Install the activity in the system"""
 
     installer = Installer(Builder(config))
-    installer.install(options.prefix, options.install_mime)
+    installer.install(options.prefix, options.install_mime, options.install_desktop_file)
+
+
+def _po_escape(string):
+    return re.sub('([\\\\"])', '\\\\\\1', string)
 
 
 def cmd_genpot(config, options):
@@ -393,6 +529,7 @@ def cmd_genpot(config, options):
                 file_path = os.path.relpath(os.path.join(root, file_name),
                                             config.source_dir)
                 python_files.append(file_path)
+    python_files.sort()
 
     # First write out a stub .pot file containing just the translated
     # activity name, then have xgettext merge the rest of the
@@ -400,16 +537,29 @@ def cmd_genpot(config, options):
     # to the end of the .pot file afterwards, because that might
     # create a duplicate msgid.)
     pot_file = os.path.join('po', '%s.pot' % config.bundle_name)
-    escaped_name = re.sub('([\\\\"])', '\\\\\\1', config.activity_name)
+    escaped_name = _po_escape(config.activity_name)
     f = open(pot_file, 'w')
     f.write('#: activity/activity.info:2\n')
     f.write('msgid "%s"\n' % escaped_name)
     f.write('msgstr ""\n')
     if config.summary is not None:
-        escaped_summary = re.sub('([\\\\"])', '\\\\\\1', config.summary)
+        escaped_summary = _po_escape(config.summary)
         f.write('#: activity/activity.info:3\n')
         f.write('msgid "%s"\n' % escaped_summary)
         f.write('msgstr ""\n')
+
+    if config.description is not None:
+        parser = HTMLParser()
+        strings = []
+        parser.handle_data = strings.append
+        parser.feed(config.description)
+
+        for s in strings:
+            s = s.strip()
+            if s:
+                f.write('#: activity/activity.info:4\n')
+                f.write('msgid "%s"\n' % _po_escape(s))
+                f.write('msgstr ""\n')
     f.close()
 
     args = ['xgettext', '--join-existing', '--language=Python',
@@ -442,6 +592,10 @@ def start():
         "--skip-install-mime", dest="install_mime",
         action="store_false", default=True,
         help="Skip the installation of custom mime types in the system")
+    install_parser.add_argument(
+        "--skip-install-desktop-file", dest="install_desktop_file",
+        action="store_false", default=True,
+        help="Skip the installation of desktop file in the system")
 
     check_parser = subparsers.add_parser(
         "check", help="Run tests for the activity")
