@@ -32,6 +32,7 @@ import dbus
 from sugar3 import env
 from sugar3 import mime
 from sugar3 import dispatch
+from sugar3.profile import get_color
 
 DS_DBUS_SERVICE = 'org.laptop.sugar.DataStore'
 DS_DBUS_INTERFACE = 'org.laptop.sugar.DataStore'
@@ -72,8 +73,6 @@ created = dispatch.Signal()
 deleted = dispatch.Signal()
 updated = dispatch.Signal()
 
-_get_data_store()
-
 
 class DSMetadata(GObject.GObject):
     """A representation of the metadata associated with a DS entry."""
@@ -224,7 +223,6 @@ class RawObject(object):
 
     def __init__(self, file_path):
         stat = os.stat(file_path)
-        settings = Gio.Settings('org.sugarlabs.user')
         metadata = {
             'uid': file_path,
             'title': os.path.basename(file_path),
@@ -232,7 +230,7 @@ class RawObject(object):
             'mime_type': Gio.content_type_guess(file_path, None)[0],
             'activity': '',
             'activity_id': '',
-            'icon-color': settings.get_string('color'),
+            'icon-color': get_color().to_string(),
             'description': file_path,
         }
 
@@ -472,6 +470,9 @@ def copy(ds_object, mount_point):
     ds_object -- DSObject to copy
     mount_point -- mount point of the new datastore entry
 
+    Returns:
+    new_ds_object -- DSObject copied
+
     """
     new_ds_object = ds_object.copy()
     new_ds_object.metadata['mountpoint'] = mount_point
@@ -492,6 +493,8 @@ def copy(ds_object, mount_point):
 
     write(new_ds_object)
 
+    return new_ds_object
+
 
 def get_unique_values(key):
     """Retrieve an array of unique values for a field.
