@@ -323,6 +323,8 @@ class HTray(Gtk.EventBox):
 
     def scroll_to_item(self, item):
         self._viewport.scroll_to_item(item)
+if hasattr(HTray, 'set_css_name'):
+    HTray.set_css_name('htray')
 
 
 class VTray(Gtk.EventBox):
@@ -414,6 +416,8 @@ class VTray(Gtk.EventBox):
 
     def scroll_to_item(self, item):
         self._viewport.scroll_to_item(item)
+if hasattr(VTray, 'set_css_name'):
+    VTray.set_css_name('VTray')
 
 
 class TrayButton(ToolButton):
